package isapi

import (
	"bytes"
	"errors"
	"io/ioutil"
	"net/http"
	"net/url"
)

const (
	contentTypeXML  = `application/xml; charset="UTF-8"`
	contentTypeJSON = `application/json; charset="UTF-8"`
	contentTypeHTML = `text/html`
)

type Client struct {
	Client  *http.Client
	BaseURL string
}

func (c *Client) Close() {
	c.Client.CloseIdleConnections()
}

func NewClient(host, username, password string) (*Client, error) {
	u, err := url.Parse("http://" + host)
	if err != nil {
		return nil, errors.New("create client error," + err.Error())
	}
	return &Client{
		Client: &http.Client{
			Transport: NewAuthTransport(username, password),
		},
		BaseURL: u.String(),
	}, nil
}

func (c *Client) Do(r *http.Request) ([]byte, error) {
	resp, err := c.Client.Do(r)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}
	return body, nil
}

func (c *Client) CommonGet(path string) (resp []byte, err error) {
	u, err := url.Parse(c.BaseURL + path)
	if err != nil {
		return nil, err
	}
	resp, err = c.Get(u)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (c *Client) CommonPut(data []byte, path string) (resp []byte, err error) {
	u, err := url.Parse(c.BaseURL + path)
	if err != nil {
		return nil, err
	}
	resp, err = c.PutXML(u, data)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (c *Client) CommonDel(data []byte, path string) (resp []byte, err error) {
	u, err := url.Parse(c.BaseURL + path)
	if err != nil {
		return nil, err
	}
	resp, err = c.DeleteXML(u, data)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (c *Client) Get(u *url.URL) ([]byte, error) {
	req, err := http.NewRequest("GET", u.String(), nil)
	if err != nil {
		return nil, err
	}
	return c.Do(req)
}

func (c *Client) put(u *url.URL, contentType string, data []byte) ([]byte, error) {
	b := bytes.NewBuffer(data)
	req, err := http.NewRequest("PUT", u.String(), b)
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", contentType)
	return c.Do(req)
}

func (c *Client) PutXML(u *url.URL, data []byte) ([]byte, error) {
	return c.put(u, contentTypeXML, data)
}

func (c *Client) PutJSON(u *url.URL, data []byte) ([]byte, error) {
	return c.put(u, contentTypeJSON, data)
}

func (c *Client) post(u *url.URL, contentType string, data []byte) ([]byte, error) {
	b := bytes.NewBuffer(data)
	req, err := http.NewRequest("POST", u.String(), b)
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", contentType)
	return c.Do(req)
}

func (c *Client) PostXML(u *url.URL, data []byte) ([]byte, error) {
	return c.post(u, contentTypeXML, data)
}

func (c *Client) PostJSON(u *url.URL, data []byte) ([]byte, error) {
	return c.post(u, contentTypeJSON, data)
}

func (c *Client) delete(u *url.URL, contentType string, data []byte) ([]byte, error) {
	b := bytes.NewBuffer(data)
	req, err := http.NewRequest("DELETE", u.String(), b)
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", contentType)
	return c.Do(req)
}

func (c *Client) DeleteXML(u *url.URL, data []byte) ([]byte, error) {
	return c.delete(u, contentTypeXML, data)
}

func (c *Client) DeleteJSON(u *url.URL, data []byte) ([]byte, error) {
	return c.delete(u, contentTypeJSON, data)
}
